package net.vidageek.mirror.invoke;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.util.Arrays;

import net.vidageek.mirror.MirrorException;

import static net.vidageek.mirror.Mirror.on;

/**
 * 
 * @author jonasabreu
 * 
 */

// TODO Must provide a way to call a constructor using classes to avoid
// ambiguity of primitive types
public class ConstructorHandler<T> {

	private final Class<T> clazz;

	public ConstructorHandler(final Class<T> clazz) {
		if (clazz == null) {
			throw new IllegalArgumentException("Argument class cannot be null");
		}
		this.clazz = clazz;
	}

	public T withoutArgs() {
		return this.withArgs(new Object[0]);
	}

	public T withArgs(final Object... args) {
		Constructor<T> constructor = this.getConstructor(args);
		try {
			return constructor.newInstance(args);
		} catch (IllegalArgumentException e) {
			throw new MirrorException("could not invoke constructor " + constructor.toGenericString() + " on class "
					+ this.clazz.getName(), e);
		} catch (InstantiationException e) {
			throw new MirrorException("could not invoke constructor " + constructor.toGenericString() + " on class "
					+ this.clazz.getName(), e);
		} catch (IllegalAccessException e) {
			throw new MirrorException("could not invoke constructor " + constructor.toGenericString() + " on class "
					+ this.clazz.getName(), e);
		} catch (InvocationTargetException e) {
			throw new MirrorException("could not invoke constructor " + constructor.toGenericString() + " on class "
					+ this.clazz.getName(), e.getCause() == null ? e : e.getCause());
		}

	}

	private Constructor<T> getConstructor(final Object... args) {
		int length = args == null ? 0 : args.length;

		Class<?>[] classes = new Class<?>[length];
		for (int i = 0; i < length; i++) {
			classes[i] = args[i].getClass();
		}

		Constructor<T> con = on(clazz).reflect().constructor().withArgs(classes);
		if (con == null) {
			throw new MirrorException("Could not find constructor with args " + Arrays.asList(classes) + " on class "
					+ this.clazz.getName());
		}

		return con;

	}

}
