package net.vidageek.mirror.invoke;

/**
 * 
 * @author jonasabreu
 * 
 */
public class InvocationHandler<T> {

	private final Object target;
	private final Class<?> clazz;

	public InvocationHandler(final Object target) {
		if (target == null) {
			throw new IllegalArgumentException("target can't be null");
		}
		this.target = target;
		clazz = target.getClass();
	}

	public InvocationHandler(final Class<T> target) {
		if (target == null) {
			throw new IllegalArgumentException("target can't be null");
		}
		clazz = target;
		this.target = null;
	}

	public MethodHandler method(final String methodName) {
		if (methodName == null) {
			throw new IllegalArgumentException("methodName can't be null");
		}
		return new MethodHandler(target, clazz, methodName);
	}

	@SuppressWarnings("unchecked")
	public ConstructorHandler<T> constructor() {
		if (this.target != null) {
			throw new IllegalStateException(
					"must use constructor InvocationHandler(Class<T>) instead of InvocationHandler(Object).");
		}
		return new ConstructorHandler<T>((Class<T>) clazz);
	}
}
