package net.vidageek.mirror.invoke;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

import net.vidageek.mirror.MirrorException;

import static net.vidageek.mirror.Mirror.on;

/**
 * 
 * @author jonasabreu
 * 
 */
// TODO Must provide a way to call a method using classes to avoid ambiguity of
// primitive types
public class MethodHandler {

	private final Object target;
	private final String methodName;
	private final Class<?> clazz;

	public MethodHandler(final Object target, final Class<?> clazz, final String methodName) {
		if (clazz == null) {
			throw new IllegalArgumentException("clazz can't be null");
		}

		if (methodName == null || methodName.trim().length() == 0) {
			throw new IllegalArgumentException("methodName can't be null");
		}

		this.target = target;
		this.clazz = clazz;
		this.methodName = methodName;
	}

	public Object withoutArgs() {
		return withArgs(new Object[0]);
	}

	public Object withArgs(final Object... args) {
		Method method = getMethod(args);

		if (target == null && !Modifier.isStatic(method.getModifiers())) {
			throw new IllegalStateException("attempt to call instance method " + methodName + " on class "
					+ clazz.getName());
		}

		try {
			return method.invoke(target, args);

		} catch (IllegalArgumentException e) {
			throw new MirrorException("Could not invoke method " + method.getName(), e);
		} catch (IllegalAccessException e) {
			throw new MirrorException("Could not invoke method " + method.getName(), e);
		} catch (InvocationTargetException e) {
			throw new MirrorException("Could not invoke method " + method.getName(), e.getCause() == null ? e : e
					.getCause());
		} catch (NullPointerException e) {
			throw new MirrorException("Attempt to call an instance method ( " + method.getName()
					+ ") on a null object.", e);
		}

	}

	private Method getMethod(final Object[] args) {
		int length = args == null ? 0 : args.length;

		Class<?>[] classes = new Class<?>[length];
		for (int i = 0; i < length; i++) {
			classes[i] = args[i].getClass();
		}

		Method method = on(clazz).reflect().method(methodName).withArgs(classes);
		if (method == null) {
			throw new MirrorException("Could not find method " + methodName + " on class " + clazz.getName());
		}
		return method;

	}

}
