/**
 * 
 */
package net.vidageek.mirror.reflect;

import java.lang.reflect.Field;

import static net.vidageek.mirror.Mirror.reflectClass;

/**
 * @author jonasabreu
 * 
 */
public class FieldReflector {

	private final String fieldName;

	public FieldReflector(final String fieldName) {
		if (fieldName == null || fieldName.trim().length() == 0) {
			throw new IllegalArgumentException("fieldname cannot be null or blank");
		}
		this.fieldName = fieldName;
	}

	public Field onClass(final String className) {
		return this.onClass(reflectClass(className));
	}

	public Field onClass(final Class<?> clazz) {
		if (clazz == null) {
			throw new IllegalArgumentException("argument clazz cannot be null.");
		}
		Class<?> c = clazz;
		Field field = null;
		while (field == null && !Object.class.equals(c)) {
			field = findField(c);
			c = c.getSuperclass();
		}
		if (field == null) {
			field = findField(Object.class);
		}
		if (field != null) {
			field.setAccessible(true);
		}
		return field;
	}

	private Field findField(final Class<?> clazz) {
		for (Field field : clazz.getDeclaredFields()) {
			if (fieldName.equals(field.getName())) {
				return field;
			}
		}
		return null;
	}
}
