/**
 * 
 */
package net.vidageek.mirror.reflect;

import java.lang.reflect.Method;

import net.vidageek.mirror.matcher.ClassArrayMatcher;
import net.vidageek.mirror.matcher.MatchType;

/**
 * @author jonasabreu
 * 
 */
public class MethodReflector {

	private final String methodName;
	private final Class<?> clazz;

	public MethodReflector(final String methodName, final Class<?> clazz) {
		if (methodName == null || methodName.trim().length() == 0) {
			throw new IllegalArgumentException("methodName cannot be null or empty");
		}
		if (clazz == null) {
			throw new IllegalArgumentException("clazz cannnot be null");
		}
		this.methodName = methodName;
		this.clazz = clazz;
	}

	public Method withoutArgs() {
		return withArgs(new Class<?>[0]);

	}

	public Method withArgs(final Class<?>... classes) {
		if (classes == null) {
			throw new IllegalArgumentException("classes cannot be null");
		}
		ClassArrayMatcher matcher = new ClassArrayMatcher(classes);
		Class<?> c = clazz;

		Method m = null;
		while (m == null && !Object.class.equals(c)) {
			m = findMethod(matcher, c);
			c = c.getSuperclass();
		}
		return m == null ? findMethod(matcher, Object.class) : m;
	}

	private Method findMethod(final ClassArrayMatcher matcher, final Class<?> c) {
		Method match = null;
		for (Method method : c.getDeclaredMethods()) {
			if (method.getName().equals(methodName)) {
				MatchType matchType = matcher.match(method.getParameterTypes());
				if (MatchType.PERFECT.equals(matchType)) {
					match = method;
					break;
				} else if (MatchType.MATCH.equals(matchType)) {
					match = method;
				}
			}
		}
		if (match != null) {
			match.setAccessible(true);
		}
		return match;
	}
}
