/**
 * 
 */
package net.vidageek.mirror.set;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;

import net.vidageek.mirror.MirrorException;

import static net.vidageek.mirror.Mirror.on;

/**
 * @author jonasabreu
 * 
 */
public class FieldSetter {

	private final String fieldName;
	private final Object target;
	private final Class<?> clazz;

	public FieldSetter(final String fieldName, final Object target, final Class<?> clazz) {
		if (fieldName == null || fieldName.trim().length() == 0) {
			throw new IllegalArgumentException("fieldName cannot be null or blank");
		}

		if (clazz == null) {
			throw new IllegalArgumentException("clazz cannot be null");
		}

		this.fieldName = fieldName;
		this.target = target;
		this.clazz = clazz;
	}

	public void withValue(final Object value) {
		Field field = on(clazz).reflect().field(fieldName);
		if (field == null) {
			throw new MirrorException("could not find field " + fieldName + " on class " + clazz.getName());
		}

		try {
			if (target == null && !Modifier.isStatic(field.getModifiers())) {
				throw new MirrorException("attempt to set instance field " + fieldName + " on class " + clazz.getName());
			}

			field.set(target, value);

		} catch (IllegalAccessException e) {
			throw new MirrorException("could not set value " + value + " on field " + fieldName + " of class "
					+ clazz.getName());
		}

	}

}
